from keras.layers \
    import \
    Conv2D, \
    GlobalAveragePooling2D

from GameVisionTargetingModel.variables.model_settings \
    import \
    get_channels


def generate_decision_middle_layer(
        layers: list
):
    decision_layer_size = 64

    layers.append(
        Conv2D(
            decision_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        Conv2D(
            decision_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        Conv2D(
            decision_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        Conv2D(
            decision_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        GlobalAveragePooling2D()
    )




from keras.layers \
    import \
    Conv2D, \
    MaxPooling2D

from GameVisionTargetingModel.variables.model_settings \
    import \
    get_channels


def generate_first_middle_layer(
        layers: list
):
    first_layer_size = 512

    layers.append(
        Conv2D(
            first_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        Conv2D(
            first_layer_size,
            get_channels(),
            padding='same',
            activation='relu'
        )
    )

    layers.append(
        MaxPooling2D(
            (2, 2)
        )
    )


# By default Settings
width: int = 512
height: int = 512

channels: int = 3

number_of_labels: int = 4
last_layer_multiplier: int = 8


# Accessors
def get_input_set():
    global \
        height, \
        width, \
        channels

    return (
        height,
        width,
        channels
    )


def get_width() -> int:
    global width
    return width


def set_width(
        value: int
) -> None:
    global width
    width = value


def get_height() -> int:
    global height
    return height


def set_height(
        value: int
) -> None:
    global width
    width = value


def get_channels() -> int:
    global channels
    return channels


def set_channels(
        value: int
) -> None:
    global channels
    channels = value


def get_last_layer_multiplier() -> int:
    global last_layer_multiplier
    return last_layer_multiplier


def get_number_of_labels() -> int:
    global number_of_labels
    return number_of_labels


def set_last_layer_multiplier(
        value: int
) -> None:
    global last_layer_multiplier
    last_layer_multiplier = value


def set_number_of_labels(
        value: int
) -> None:
    global number_of_labels
    number_of_labels = value



rescaling_multiplier: float = 1./255


def get_rescaling_multiplier() -> float:
    global rescaling_multiplier
    return rescaling_multiplier


def set_rescaling_multiplier(value: float) -> None:
    global rescaling_multiplier
    rescaling_multiplier = value



from GameVisionTargetingModel.variables.model_settings \
    import                  \
    get_input_set,          \
    get_width,              \
    get_channels,           \
    get_height,             \
    get_number_of_labels,   \
    get_last_layer_multiplier

from GameVisionTargetingModel.variables.model_settings \
    import                  \
    set_width,              \
    set_height,             \
    set_channels,           \
    set_number_of_labels,   \
    set_last_layer_multiplier

from GameVisionTargetingModel.variables.normalise_settings \
    import \
    get_rescaling_multiplier, \
    set_rescaling_multiplier



from keras.models \
    import Sequential

from tensorflow import compat, get_logger

from GameVisionTargetingModel.models.classification.map.v1.layers \
    import generate_layers_for_map_classifier_v1

from keras.losses \
    import SparseCategoricalCrossentropy

from keras.metrics              \
    import                      \
    Accuracy,                   \
    SparseCategoricalAccuracy

from logging \
    import ERROR


def tensorflow_settings():
    compat.v1.disable_eager_execution()
    get_logger().setLevel(
        ERROR
    )


class MapClassifier(
    Sequential
):
    def __init__(self):
        super().__init__(
            generate_layers_for_map_classifier_v1()
        )
        tensorflow_settings()

        self.setup()

    def setup(self):
        self.compile(
            loss=SparseCategoricalCrossentropy(
                from_logits=True,
                ignore_class=None
            ),
            metrics=[
                Accuracy('accuracy'),
            ],
            optimizer='adam',
            jit_compile=True
        )




from GameVisionTargetingModel.models.classification.map.v1.setup.middle_layer.first \
    import generate_first_middle_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.middle_layer.second \
    import generate_second_middle_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.middle_layer.third \
    import generate_third_middle_layer

from GameVisionTargetingModel.models.classification.map.v1.setup.middle_layer.decision \
    import generate_decision_middle_layer




from GameVisionTargetingModel.setup_package \
    import default_settings

from GameVisionTargetingModel.variables \
    import          \
    get_width,      \
    get_height,     \
    get_channels


def test_defaults() -> None:
    default_settings()


def test_height() -> None:
    assert get_height() == 512


def test_width() -> None:
    assert get_width() == 512


def test_channels() -> None:
    assert get_channels() == 3



from keras.layers \
    import \
    Rescaling

from GameVisionTargetingModel.variables \
    import                              \
    get_input_set,                      \
    get_rescaling_multiplier


def generate_input_layer(
    layers: list
):
    layers.append(
        Rescaling(
            get_rescaling_multiplier(),
            input_shape=get_input_set()
        )
    )

